	/**********************************************************************/
	/*           PROCESOR ARYTMII / POSTPROCESOR DLA HOLTERA              */
	/*                                                                    */
	/**********************************************************************/

/* PROCEDURY ZAWARTE W TYM MODULE:
			void RrAll(INT_L QrsMaxNr)
			void ArhDispl(INT_S ile)
			INT_S ArhMan(INT_S cmd, INT_L CurQrsNr, INT_S ArhT, INT_S ArhT0)
			void ArhyDet(INT_L QrsMaxNr)
			void Rhythm_Ev_Det(INT_L QrsMaxNr)
*/

#include <stdio.h>
#include "holpa.h"
#include "holpaext.h"
#include "holpagdf.h"
#include "holdw.h"

#define BDHR 255 // znacznik "brak danej HR" 

/* wywolywane procedury zawarte w innych modulach */
extern void STEvents(INT_L QrsMaxNr); //w h_st.cpp
extern void MorphReOrd(INT_L QrsMaxNr); //w h_morph.cpp
extern void HRV_mgr(INT_L StartQrsNr, INT_L EndQrsNr); //w h_hrv.cpp
/* ------------------------------------------------------------------- */



	/**********************************************************************/
	/*             OBLICZANIE PROCENTOWEGO UDZIALU ARTEFAKTOW             */
	/*                                                                    */
	/**********************************************************************/

void ArtfContr(INT_L StartQrsNr, INT_L EndQrsNr)
/* funkcja realizuje obliczenie procentowego udzialu artefaktow i odcinkow	*/
/* nieklasyfikowanych w zapisie holterowskim	(tablica PTab)								*/
/*                                                              						*/
/* parametry wejsciowe: QrsMaxNr - ilosc zespolow QRS w tablicy PTab				*/
/* PA 19.11.98 v. 1.09																											*/
 
	{
// czesciowe zerowanie struktury wynikowej 
	TimeDescr.ArtSig=0;
	TimeDescr.ValSig=0;

// jezeli przetwarzanie od poczatku pliku
	if(StartQrsNr<1)
		{
		StartQrsNr=1;
		TimeDescr.ArtSig=PTab[0].DetPt; // poczatkowy odcinek zaliczany jest do artefaktow 
		}

// obliczenia w petli
	for (CurQrsNr=StartQrsNr; CurQrsNr<EndQrsNr; CurQrsNr++)
		{
		if(PTab[CurQrsNr].MorfT==4 || (PTab[CurQrsNr].MorfT==3 && PTab[CurQrsNr].cl<0)) 
			TimeDescr.ArtSig+=(PTab[CurQrsNr].DetPt-PTab[CurQrsNr-1].DetPt);
		else
			TimeDescr.ValSig+=(PTab[CurQrsNr].DetPt-PTab[CurQrsNr-1].DetPt); 			
		}

// parametry koncowe
	TimeDescr.TotSig=TimeDescr.ArtSig+TimeDescr.ValSig;
	TimeDescr.PercArtf=(TimeDescr.ArtSig<<6)/(TimeDescr.TotSig>>4);
	TimeDescr.QrsMaxNr=EndQrsNr-StartQrsNr;
	SendMessage(hKernelWnd, AHM_TIMESTAT, NULL, (LPARAM)&TimeDescr); // (*DW*)
	}
/* ------------------------------------------------------------------- */



	/**********************************************************************/
	/*             OKRESLANIE ODSTEPU R-R I CZESTOSCI RYTMU HR            */
	/*                                                                    */
	/**********************************************************************/

void RrAll(INT_L QrsMaxNr)
/* funkcja realizuje obliczenia interwalu miedzyskurczowego serca (r-r) */
/* na podstawie kolejnych punktow maksymalnych QRS; sposrod interwalow  */
/* odrzucany jest najdluzszy i dwa najkrotsze a pozostale sa usredniane */
/*                                                              				*/
/* parametry wejsciowe: QrsMaxNr - ilosc zespolow QRS w tablicy PTab    */
/* funkcja zwraca indeks najszybszego rytmu HR											*/
/* PA 15.10.98 v. 1.09																									*/

	{
	INT_S QrsDist[32]={0}; // bufor ostatnich 32 detekcji qrs (dla oblicz. r-r)
	INT_S ptr=0;
	INT_L lastqrs=(PTab[0].DetPt<<3)+PTab[0].AMax;
	INT_L sum=0;
	INT_S i=0;         // zmienna iteracyjna
	INT_S min1=0x7FFF; // pierwsza wartosc minimalna
	INT_S x1=0;        // polozenie pierwszej wartosci minimalnej
	INT_S min2=0x7FFF; // druga wartosc minimalna
	INT_S x2=0;        // polozenie drugiej wartosci minimalnej
	INT_S max=0;       // wartosc maksymalna
	INT_S x3=0;        // polozenie wartosci maksymalnej
	INT_L ptQ=0;       // dokladne polozenie maksimum biezacego QRS
	INT_L sumN=0;      // wartosc tymczasowa sumy bufora
	INT_C SVCont=0;		 // licznik kolejnych zespolow nadkomorowych  
	INT_S BufValid=0;  // znacznik waznosci danych w buforze
	INT_S QrsMarg=Cfg[CurCfg].cfge.STEvHRDist+DROPQRS; 

	for (CurQrsNr=1; CurQrsNr<QrsMaxNr; CurQrsNr++)
		{
		// obliczenia .localRR
		ptQ=(INT_L)(((double)(PTab[CurQrsNr].DetPt)*7.8125)+PTab[CurQrsNr].AMax+0.5);
		if((PTab[CurQrsNr].localRR = (INT_S)(ptQ-lastqrs))==0)
			continue;
		lastqrs=ptQ;
		// obliczenia 
		if(((PTab[CurQrsNr].MorfT==-1) && (PTab[CurQrsNr].cl == -8)) 
			/*|| ((Cfg[CurCfg].cfga.HRateOnArtf==0) && (PTab[CurQrsNr].cl== -5))*/)
			// odcinki wylaczone z klasyfikacji przez operatora 
			// na odcinkach uznanych przez program za artefakty obliczanie HR jest dozwolone
			{
			BufValid=0;
			ptr=0;
			sum=0; // zerowanie wartosci sumy 
			// memset(QrsDist, 0, 64); // zerowanie bufora i wartosci sumy 
			// zerowanie bufora nie jest konieczne, jezeli wstrzymane zostanie odejmowanie od sumy do czasu wypelnienia bufora
			PTab[CurQrsNr].HR=BDHR; // znacznik "brak danej HR" 
			}
		else
			{ 			
			if(BufValid)
				{
				sum -= QrsDist[ptr]; // wstrzymanie odejmowania od sumy do czasu wypelnienia bufora
				sum += (QrsDist[ptr] = PTab[CurQrsNr].localRR);
				min1=0x7FFF;
				min2=0x7FFF;
				max=0;
				for(i=0; i<Cfg[CurCfg].cfga.HRateQRSCnt; i++)
					if(QrsDist[i] < min1)
						min1=QrsDist[(x1=i)];
				for(i=0; i<Cfg[CurCfg].cfga.HRateQRSCnt; i++)
					if((QrsDist[i] < min2) && (i!=x1))
						min2=QrsDist[(x2=i)];
				for(i=0; i<Cfg[CurCfg].cfga.HRateQRSCnt; i++)
					if((QrsDist[i] > max) && (i!=x1) && (i!=x2))
						max=QrsDist[(x3=i)];
				if(QrsDist[x1])
					{
					sumN=sum-(QrsDist[x1]+QrsDist[x2]+QrsDist[x3])+4;
					PTab[CurQrsNr].HR=(UINT_C)(((sumN>>1)+60000L*(Cfg[CurCfg].cfga.HRateQRSCnt-3))/sumN);
					}
				else
					PTab[CurQrsNr].HR=(UINT_C)(60000l/PTab[CurQrsNr].localRR);				
				}
			else
				// uzupelnianie bufora, HR obliczany bez usredniania (z wyjatkiem pierwszego zespolu) 
				{
				sum += (QrsDist[ptr] = PTab[CurQrsNr].localRR);
				if(ptr>1)
					{
					// usrednianie tej czesci bufora, ktora jest wypelniona
					sumN=(sum-QrsDist[0])/ptr;
					PTab[CurQrsNr].HR=(UINT_C)(60000l/sumN);
					}
				else
					PTab[CurQrsNr].HR=BDHR; // znacznik "brak danej HR" 
				}
			if(++ptr == Cfg[CurCfg].cfga.HRateQRSCnt)
				{
				ptr=0;
				if(!BufValid) // caly bufor jest juz wypelniony
					BufValid=1; 
				}

			} // koniec else 
		} // koniec for(...
	PTab[0].localRR=PTab[1].localRR;
	PTab[0].HR=PTab[1].HR;
	}

/* ------------------------------------------------------------------- */



	/**********************************************************************/
	/*                          DETEKCJA ARYTMII                          */
	/*                                                                    */
	/**********************************************************************/

void ArhDispl(INT_S ile)
/* procedura testowa wyswietlajaca wykryte arytmie */
	{
	INT_S err=0; // blad uzycia struktury
	INT_S ArhT=0;
	ARH_ATTR& arh= *Arh;
	printf("\n");
	for(ArhT=1; ArhT<13; ArhT++)
		{
		printf("%d  ",ArhT);
		for(INT_S j=0; j<ile; j++)
			printf("%ld\t",arh.get(ArhT, j, &err).Beg);
		printf("\n");
		}
	}

INT_S ArhMan(INT_S cmd, INT_L CurQrsNr, INT_S ArhT, INT_S ArhT0)
/* Manager arytmii - dokonuje otwierania, zamykania, zmiany nazwy, 		*/
/* i rejestrowania arytmii w strukturze SArh 													*/
/* parametry wejsciowe:																								*/
/* cmd - polecenie: 1 - otworz arytmie,																*/
/*                  2 - zamknij arytmie,                    					*/
/*									3 - zmien nazwe arytmii                 					*/
/* 									4 - zarejestruj arytmie                 					*/
/* 									5 - zeruj liczniki arytmii              					*/
/* CurQrsNr - numer biezacego zespolu QRS															*/
/* ArhT numer arytmii (zrodlowy numer arytmii) 												*/
/* ArhT numer arytmii (docelowy numer arytmii) 												*/
/* funkcja zwraca numer komunikatu bledu ErrType:               			*/
/* 									0 - OK.                                           */
/* 									1 - nieznane polecenie                            */
/* 									2 - proba otwarcia niedozwolonej liczby arytmii   */
/* 									3 - proba zamkniecia nieotwartej arytmii          */
/* 									4 - niewlasciwe polecenie dla danego typu arytmii */
/* 									5 - proba przeniesienia zamknietej arytmii        */
/* PA 22.07.98   v. 1.08                                  						*/

	{
	static INT_L ACnt[16]={0}; // licznik epizodow arytmii poszczegolnych typow
	INT_S ErrType=0; // numer komunikatu o bledzie
	INT_S err=0; // blad uzycia struktury
	ARH_ATTR& arh= *Arh;
	INT_S minRr4=4000; // wartosc poczatkowa min z 4 RR
	INT_S maxRr4=0;    // wartosc poczatkowa max z 4 RR
	INT_S sumRr=0;     // suma/srednia 4 kolejnych localRR
	INT_S bufRr[4]={0}; // bufor 4 kolejnych wartosci localRR
	INT_S bufRrPtr=0;  // wskaznik w buforze bufRr
	INT_L TmpQrsNr=0; // tymczasowy numer zespolu QRS

	switch(cmd)
		{
		case 1: //open (dotyczy typow: 2, 3, 4, 6, 8, 9, 10, 11)
			if((ArhT==2) || (ArhT==3) ||  (ArhT==4) ||  (ArhT==6) ||  (ArhT==8) ||
	 (ArhT==9) || (ArhT==10) ||  (ArhT==11))
				{
				arh.get(ArhT, ACnt[ArhT], &err).Beg=CurQrsNr;
				if(err > 0)
					{
					arh.ErrFlag |= (0x0001 << (err-1));
					ErrType=2; // proba otwarcia niedozwolonej liczby arytmii
					}
				}
			else
				ErrType=4; // niewlasciwe polecenie dla danego typu arytmii
			break;

		case 2: //close (dotyczy typow: 2, 3, 4, 6, 8, 9, 10, 11)
			if((ArhT==2) || (ArhT==3) ||  (ArhT==4) ||  (ArhT==6) ||  (ArhT==8) ||
	 (ArhT==9) || (ArhT==10) ||  (ArhT==11))
				{
				if(arh.get(ArhT, ACnt[ArhT], &err).Beg > 0)
				// sprawdz czy otwarto arytmie
					{
				// czas trwania w sekundach
					arh.get(ArhT, ACnt[ArhT], &err).Dur=(PTab[CurQrsNr].DetPt-PTab[arh.get(ArhT, ACnt[ArhT], &err).Beg-1].DetPt+64)>>7;
				// ilosc zespolow w sekwencji
					arh.get(ArhT, ACnt[ArhT], &err).BNr=(INT_S)(CurQrsNr-arh.get(ArhT, ACnt[ArhT], &err).Beg+1);
				// srednia czestosc rytmu
					if(arh.get(ArhT, ACnt[ArhT], &err).Dur>0)
						{
						INT_L DurSmp=(PTab[CurQrsNr].DetPt-PTab[arh.get(ArhT, ACnt[ArhT], &err).Beg].DetPt)
							+((PTab[CurQrsNr].AMax-PTab[arh.get(ArhT, ACnt[ArhT], &err).Beg].AMax)>>3); // Czas trwania w ilosci probek
						arh.get(ArhT, ACnt[ArhT], &err).Hr0=
							(INT_C)(((7680l*(arh.get(ArhT, ACnt[ArhT], &err).BNr-1))+(DurSmp >> 1))/DurSmp);
						}
					else
						{
						arh.get(ArhT, ACnt[ArhT], &err).Hr0=
							(PTab[arh.get(ArhT, ACnt[ArhT], &err).Beg].HR+PTab[CurQrsNr].HR)>>1;							
						}
					if((ArhT==8) || (ArhT==9) ||  (ArhT==10))
					// przegladanie Hr4
						{
						for(INT_L i=arh.get(ArhT, ACnt[ArhT], &err).Beg-2;  i<CurQrsNr+2; i++)
							{
							if((i > arh.get(ArhT, ACnt[ArhT], &err).Beg-2) && (i <= arh.get(ArhT, ACnt[ArhT], &err).Beg+2))
								{
								// srednia z 1, 2, 3 oraz pierwsza srednia z 4 kolejnych rr
								switch (i-(arh.get(ArhT, ACnt[ArhT], &err).Beg-2))
									{
									case 1: 
										maxRr4=minRr4=sumRr;								
										break;
									case 2: 
										maxRr4=minRr4=sumRr>>1;								
										break;
									case 3: 
										maxRr4=minRr4=sumRr/3;								
										break;
									case 4:
										maxRr4=minRr4=sumRr>>2;
										sumRr-=bufRr[bufRrPtr];
										break;
									}
								}
							else if(i > arh.get(ArhT, ACnt[ArhT], &err).Beg+2)
								// kolejna srednia z 4 kolejnych rr
								{
								INT_S TsumRr=sumRr>>2;
								if(TsumRr < minRr4)
									minRr4=TsumRr;
								if(TsumRr > maxRr4)
									maxRr4=TsumRr;
								sumRr-=bufRr[bufRrPtr];
								}
							sumRr+=PTab[i].localRR;
							bufRr[bufRrPtr++]=PTab[i].localRR;
							bufRrPtr &= 0x0003;
							}
						if(ArhT==9)
						// minimalna wartosc HR liczona z 4 kolejnych interwalow
							arh.get(ArhT, ACnt[ArhT], &err).Hr4=(INT_C)(60000l/maxRr4);
						else
						// maksymalna wartosc HR liczona z 4 kolejnych interwalow
							arh.get(ArhT, ACnt[ArhT], &err).Hr4=(INT_C)(60000l/minRr4);
						}
					if(ArhT==8)
						// anulowanie wszystkich zespolow przedwczesnych w obrebie PSVT 
						{
						// wycofanie atrybutu przedwczesnosci dla zespolow poczatkowych PSVT
						for(TmpQrsNr=CurQrsNr; TmpQrsNr>arh.get(ArhT, ACnt[ArhT], &err).Beg-3; TmpQrsNr--)
							if(PTab[TmpQrsNr].MorfT==1)
								PTab[TmpQrsNr].MorfT=0;
						// skasowanie zespolow przedwczesnych na liscie arytmii S'SVE (typu 1) (ArhT=8)
						ARH_ATTR_ITEM* pItem;
						while(ACnt[1]>0) // dopoki sa na liscie zespoly przedwczesne
							if(arh.get(1, ACnt[1]-1, &err).Beg > arh.get(ArhT, ACnt[ArhT], &err).Beg-3) 
								// ostatnia z zespolow przedwczesnych jest pozniejszy niz poczatek PSVT
								{								
								ACnt[1]--;
								pItem= arh[0][ACnt[1]]; // UWAGA na przekodowanie ponizej 
								arh[0].RemoveAt(ACnt[1]); // (opisane w holpa.inl)
								delete pItem;							
									// Wymagana kolejnosc arytmi w tablicy ArhArray
									// se=0, sp=1, st=2, br=3, si=4, pz=5, ve=6, vt=7, vr=8, vs=9,	bi=10, pr=11,	
									// Kody arytmi stosowane przez PA
									// 1-se, 2-vt, 3-vr, 4-vs, 5-pr, 6-bi, 7-ve, 8-sp, 9-br, 10-st, 11-si, 12-pz, 
								}
							else
								break;
						}
					ACnt[ArhT]++;
					}
				else
					ErrType=3; // proba zamkniecia nieotwartej arytmii
				}
			else
				ErrType=4; // niewlasciwe polecenie dla danego typu arytmii
			break;

		case 3: //rename (dotyczy typow: 2, 3, 4)
			if(((ArhT==2) || (ArhT==3) ||  (ArhT==4)) && ((ArhT0==2) || (ArhT0==3) ||  (ArhT0==4)))
				{
				// sprawdz czy arytmia zrodlowa ArhT nie jest zamknieta
				if(arh.get(ArhT, ACnt[ArhT], &err).Dur <= 0)
					{
					INT_L BegTmp=arh.get(ArhT, ACnt[ArhT], &err).Beg;
					// sprawdz czy arytmia docelowa ArhT0 moze zostac otwarta
					arh.get(ArhT0, ACnt[ArhT0], &err).Beg=BegTmp;
					if(err > 0)
						{
						arh.ErrFlag |= (0x0001 << (err-1));
						ErrType=2; // proba otwarcia niedozwolonej liczby arytmii
						}
					else
						// przepisanie wskaznika poczatku z arytmii ArhT do ArhT0
						// przebieglo pomyslnie - mozna skasowac wartosc w ArhT
						arh.get(ArhT, ACnt[ArhT], &err).Beg=-1;
					}
				else
					ErrType=5; // proba przeniesienia zamknietej arytmii
				}
			else
				ErrType=4; // niewlasciwe polecenie dla danego typu arytmii
			break;

		case 4: //register (dotyczy typow: 1, 5, 7, 12)
			if((ArhT==1) || (ArhT==5) ||  (ArhT==7) ||  (ArhT==12))
				{
				arh.get(ArhT, ACnt[ArhT], &err).Beg=CurQrsNr;
				if(err > 0)
					{
					arh.ErrFlag |= (0x0001 << (err-1));
					ErrType=2; // proba otwarcia niedozwolonej liczby arytmii
					}
				if(ArhT==12)
				// dla pauzy - dlugosc w ms podana w parametrze ArhT0
					arh.get(ArhT, ACnt[ArhT], &err).Dur=ArhT0; 
				ACnt[ArhT]++;
				}
			else
				ErrType=4; // niewlasciwe polecenie dla danego typu arytmii
			break;

		case 5: //reset
			for (ArhT=0; ArhT<16; ArhT++)
				ACnt[ArhT]=0;
			break;

		default:
			ErrType=1; // nieznane polecenie
			break;
		}

	ArhStatus.SVTotal=ACnt[1]+ACnt[8]+ACnt[10]+ACnt[11];
			// arytmie nadkomorowe: pojed. nadkom. pob. przedwczesne, PSVT,
			// 	tachykardia nadkomorowa i rytm nieregularny
	ArhStatus.VTotal=ACnt[2]+ACnt[3]+ACnt[4]+ACnt[5]+ACnt[7];
			// arytmie komorowe: pojedyncze komorowe, czynny rytm komor, salvo,
			//	tachykardia komorowa i pary
	ArhStatus.ErrMsgNr=ErrType; //nr komunikatu o bledzie
	return ErrType;
	}

void ArhyDet(INT_L QrsMaxNr)
/* funkcja realizuje detekcje arytmii na podstawie danych czasowych i   */
/* morfologicznych tablicy PTab[]                                       */
/*                                                              				*/
/* parametry wejsciowe: QrsMaxNr - ilosc zespolow QRS w tablicy PTab    */
/* PA 17.02.97	v. 1.08																									*/
	{
	INT_S Flags[16]={0}; //znaczniki rodzajow arytmii:
	// 0 - brak arytmii / oczekiwanie poczatku
	// 1..9 - rozpoczeta detekcja poczatku
	// 10 - stwierdzono poczatek / kontrola parametrow / oczekiwanie konca
	// 11..19 - rozpoczeta detekcja konca
	// 20 - stwierdzono koniec, ale nie zapisano parametrow
	// 0 - ....
	INT_L DBseq=0; // ilosc kolejnych pobudzen dominujacych
	INT_L SVseq=0; // ilosc kolejnych pobudzen nadkomorowych
	INT_L VEseq=0; // ilosc kolejnych pobudzen komorowych
	INT_L PSVTHRCnt=0; // ilosc kolejnych pobudzen spelniajacych warunki poczatku/konca PSVT
	INT_L SVTHRCnt=0; // ilosc kolejnych pobudzen spelniajacych warunki poczatku/konca ST
  INT_L IRRCnt=0;   // ilosc kolejnych zespolow QRS spelniajacych warunki poczatku/konca IRR
  INT_L PrvQrsNr=0; // poprzedni numer zespolu QRS (test ciaglosci)
  INT_S SVHR=0; // czestosc skurczow liczona tylko dla zespolow nadkomorowych
  INT_S HRBuff[32]={0}; // bufor ostatnich wartosci HR zespolow dominujacych
  INT_S HRBuffP=0; // wskaznik w buforze HRBuff
  INT_L sum=0; //suma/srednia wartosc RR
	INT_L LastPauseAt=0; // znacznik konca ostatnio wykrytej pauzy

  ArhMan(5, 0, 0, 0); //reset
  for (CurQrsNr=DROPQRS+1; CurQrsNr<QrsMaxNr; CurQrsNr++)
    {
		if(PTab[CurQrsNr].DetPt==0)
			break;
		if((PTab[CurQrsNr].MorfT==-1) && (PTab[CurQrsNr].cl != -8))
			PTab[CurQrsNr].MorfT=3; /// okreslone automatycznie zespoly "nie okreslone" przechodza w "inne"
		if(PTab[CurQrsNr].cl>-1)
			{
			if((PTab[CurQrsNr].MorfT==1) && (ClRep[PTab[CurQrsNr].cl][9]))
				PTab[CurQrsNr].MorfT=0; /// zespoly "nadkomorowe" przechodza w "dominujace"
			}
    if(!(CurQrsNr & 0x03FF)) // co 1024 zespol QRS
			{
      /* uzupelnienie struktury ArhStatus */
      ArhStatus.CurQrsNr=CurQrsNr;
      Anuluj_Button= (INT_C)SendMessage(hKernelWnd, AHM_ARYTHM, NULL, (LPARAM)&ArhStatus); // (*DW*)
      /* sprawdzenie czy nie nacisnieto klawisza "Anuluj" */
      if(Anuluj_Button)
				{
				ArhStatus.ErrMsgNr=-3; //user break
				break;
				}
      }
		/* odrzucenie zespolow o morfologii nieokreslonej */
		if ((PTab[CurQrsNr].MorfT==-1) && (Cfg[CurCfg].cfga.BrkOnNoClass==1))
		// przerwanie arytmii po napotkaniu zespolu nie klasyfikowanego
			{
			VEseq=0;
			SVseq=0;
			DBseq=0;
			HRBuffP=0;
			PSVTHRCnt=0;
			SVTHRCnt=0;
			IRRCnt=0;
			for(INT_S i=0; i<16; i++)
				{
				if(Flags[i]==10)
					ArhMan(2, CurQrsNr, i, i); //close i
				Flags[i]=0;
				}
			continue;
			}

		/* odrzucenie zespolow oznaczonych jako "inne" */
		if ((PTab[CurQrsNr].MorfT==3) && (Cfg[CurCfg].cfga.BrkOnUnkn==1))
		// przerwanie arytmii po napotkaniu zespolu o morfologii nieznanej
			{
			VEseq=0;
			SVseq=0;
			DBseq=0;
			HRBuffP=0;
			PSVTHRCnt=0;
			SVTHRCnt=0;
			IRRCnt=0;
			for(INT_S i=0; i<16; i++)
				{
				if(Flags[i]==10)
					ArhMan(2, CurQrsNr, i, i); //close i
				Flags[i]=0;
				}
			continue;
			}

		/* odrzucenie zespolow oznaczonych jako artefakty */
		if(((PTab[CurQrsNr].MorfT==4) || (PTab[CurQrsNr].MorfT==5)) && (Cfg[CurCfg].cfga.BrkOnArtf==1))
		// przerwanie arytmii po napotkaniu artefaktu
			{
			VEseq=0;
			SVseq=0;
			DBseq=0;
			HRBuffP=0;
			PSVTHRCnt=0;
			SVTHRCnt=0;
			IRRCnt=0;
			for(INT_S i=0; i<16; i++)
				{
				if(Flags[i]==10)
					ArhMan(2, CurQrsNr, i, i); //close i
				Flags[i]=0;
				}
			continue;
			}

		/* odrzucenie zespolow oznaczonych jako migotanie komor */
		if ((PTab[CurQrsNr].MorfT==6) && (Cfg[CurCfg].cfga.BrkOnVFib==1))
		// przerwanie arytmii po napotkaniu migotania komor
			{
			VEseq=0;
			SVseq=0;
			DBseq=0;
			HRBuffP=0;
			PSVTHRCnt=0;
			SVTHRCnt=0;
			IRRCnt=0;
			for(INT_S i=0; i<16; i++)
				{
				if(Flags[i]==10)
					ArhMan(2, CurQrsNr, i, i); //close i
				Flags[i]=0;
				}
			continue;
			}

		/* ++++++++ GRUPA ARYTMII NIE UWZGLEDNIAJACYCH MORFOLOGII ++++++++ */
		/* --------------------------------------------------------------- */

		/* ===== pauzy ===== */
		INT_S l=PTab[CurQrsNr].localRR;
		INT_S i=1;
		if(Cfg[CurCfg].cfga.PseArtfF)
			{
			while((PTab[CurQrsNr-i].cl==-4) || (PTab[CurQrsNr-i].cl==-5) ||
					(PTab[CurQrsNr-i].MorfT==4) || (PTab[CurQrsNr-i].MorfT==5))
				{
				l+=PTab[CurQrsNr-i].localRR;
				i++;
				}
			}
		switch(PTab[CurQrsNr-i].MorfT)
			{
			case 0: // ostatni prawidlowy zespol jest dominujacy
				if((l > Cfg[CurCfg].cfga.PseMinDB) && (Cfg[CurCfg].cfga.PseMinDB > -1))
					Flags[12]=10;
				break;

			case 1: // ostatni prawidlowy zespol jest nadkomorowy
				if((l > Cfg[CurCfg].cfga.PseMinSV) && (Cfg[CurCfg].cfga.PseMinSV > -1))
					Flags[12]=10;
				break;

      case 2: // ostatni prawidlowy zespol jest komorowy
				if((l > Cfg[CurCfg].cfga.PseMinVE) && (Cfg[CurCfg].cfga.PseMinVE > -1))
					Flags[12]=10;
				break;

      default:	break; // po zespolach o innej morfologii pauzy nie sa wykrywane
      }
    if(Flags[12] > 0)
      {
      if(Flags[8] > 0)
				{
				// operacje zwiazane z zakonczeniem PSVT
				ArhMan(2, CurQrsNr, 8, 8); //close 8
				Flags[8]=0; // zakonczenie ewentualnego PSVT
				}
      // operacje zwiazane z zapisem pauzy o dlugosci l
      ArhMan(4, CurQrsNr, 12, l); //register 12
			LastPauseAt=CurQrsNr; // zaznaczenie pauzy dla celow detekcji przedwczesnosci
			PTab[CurQrsNr].HR=(PTab[CurQrsNr-1].HR+PTab[CurQrsNr+1].HR)>>1;
      Flags[12]=0;
      }

    /* ===== bradykardia ===== */
    // bradykardia nie zwraca uwagi na morfologie
    INT_S hrate=(INT_S)(60000l/PTab[CurQrsNr].localRR);
    switch(Flags[9])
			{
			case 0:
				if(hrate < Cfg[CurCfg].cfga.BraMaxHRtoStart)
					Flags[9]=1; // rozpoczeta detekcja poczatku bradykardii
				break;

      case 1:
				if(hrate < Cfg[CurCfg].cfga.BraMaxHRtoStart)
					Flags[9]=2; // rozpoczeta detekcja poczatku bradykardii
				else
					Flags[9]=0; // porzucenie detekcji poczatku bradykardii
				break;

      case 2:
				if(hrate < Cfg[CurCfg].cfga.BraMaxHRtoStart)
					{
					Flags[9]=10; // stwierdzono poczatek bradykardii
					if(Flags[8]>0)
						{
						// operacje zwiazane z zakonczeniem PSVT
						ArhMan(2, CurQrsNr, 8, 8); //close 8
						Flags[8]=0; // zakonczenie ewentualnego PSVT
						}
					// operacje zwiazane z inicjacja bradykardii
					ArhMan(1, CurQrsNr, 9, 9); //open 9
					}
				else
					Flags[9]=0; // porzucenie detekcji poczatku bradykardii
				break;

      case 10:
				if(hrate > Cfg[CurCfg].cfga.BraMinHRtoStop)
					Flags[9]=11; // rozpoczeta detekcja konca bradykardii
				break;

      case 11:
				if(hrate > Cfg[CurCfg].cfga.BraMinHRtoStop)
					Flags[9]=12; // rozpoczeta detekcja konca bradykardii
				else
					Flags[9]=10; // porzucenie detekcji konca bradykardii
				break;

      case 12:
				if(hrate > Cfg[CurCfg].cfga.BraMinHRtoStop)
					{
					Flags[9]=20; // stwierdzono koniec bradykardii
					// operacje zwiazane z zamykaniem bradykardii
					ArhMan(2, CurQrsNr, 9, 9); //close 9
					}
				else
					Flags[9]=10; // porzucenie detekcji konca bradykardii
				break;

			case 20:
				Flags[9]=0;
			break;

      default: break;
      }

    /* ===== poszukiwanie konca tachykardii nadkomorowej ===== */
    // rytm dowolnego pochodzenia o czestosci podprogowej
    if(Flags[10]==10)
      {
      if((CurQrsNr==PrvQrsNr+1) && ((INT_S)(60000l/PTab[CurQrsNr].localRR) < Cfg[CurCfg].cfga.SVTMaxHRtoStop))
				SVTHRCnt++;
      else
				SVTHRCnt=0;
      if(SVTHRCnt > 2)
				{
				// koniec tachykardii nadkomorowej
				ArhMan(2, CurQrsNr, 10, 10); // close 10
				Flags[10]=0;
				SVTHRCnt=0;
				}
      }
    /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

    /* ++++++++++++++++ GRUPA ARYTMII MORFOLOGICZNYCH ++++++++++++++++ */
		/* --------------------------------------------------------------- */

		/* ===== rozroznienie morfologii zespolu ===== */
    if (PTab[CurQrsNr].MorfT==0 || (PTab[CurQrsNr].MorfT==1))
      // zespol jest nadkomorowy
      {
      SVseq++;
      VEseq=0;

      // obliczenia HR dla zespolow nadkomorowych
      if ((PTab[CurQrsNr].MorfT==0) || (PTab[CurQrsNr].MorfT==1))
				{
				if (DBseq >= Cfg[CurCfg].cfga.PremRRCnt)
					{
					SVHR=(INT_S)((sum+(Cfg[CurCfg].cfga.PremRRCnt>>1))/Cfg[CurCfg].cfga.PremRRCnt);
					sum-=HRBuff[HRBuffP];
					}
				else
					SVHR=0;
				if(DBseq==0)
					sum=0;
				sum+=PTab[CurQrsNr].localRR;
				HRBuff[HRBuffP]=PTab[CurQrsNr].localRR;
				DBseq++;
				if(++HRBuffP==Cfg[CurCfg].cfga.PremRRCnt)
					HRBuffP=0;
				}

      // skasowanie lub zakonczenie ew. arytmii komorowych
			if(Flags[2]==10)
				{
				ArhMan(2, PrvQrsNr, 2, 2); // close VT
				Flags[2]=0;
				}
      if(Flags[3]==10)
				{
				ArhMan(2, PrvQrsNr, 3, 3); // close VR
				Flags[3]=0;
				}
      if(Flags[4]==10)
				{
				ArhMan(2, PrvQrsNr, 4, 4); // close VS
				Flags[4]=0;
				}

      /* ===== bigeminia (czesc A) ===== */
      if(SVseq==1)
				{
				if ((PTab[CurQrsNr+1].MorfT==0) || (PTab[CurQrsNr+1].MorfT==1))
		// VNN...
					{
					if(Flags[6]==10)
			// zakonczenie bigeminii
					ArhMan(2, CurQrsNr, 6, 6);
					if((Flags[6]==2) && (PTab[CurQrsNr-1].MorfT==2))
						ArhMan(4, CurQrsNr-1, 7, 7);   // poprzedni QRS jest VE
					Flags[6]=0;
					}
				else if(PTab[CurQrsNr+1].MorfT==2)
					{
					if ((PTab[CurQrsNr+2].MorfT==0) || (PTab[CurQrsNr+2].MorfT==1))
				// VNVN
						{
						switch(Flags[6]) //wskaznik bigeminii
							{
							case 0: Flags[6]=2;
								break;
							case 2: Flags[6]=10;
								ArhMan(1, CurQrsNr-2, 6, 6);
								break;
							}
						}
					else if (PTab[CurQrsNr+2].MorfT==2)
						{
					// VNVV
						if((Flags[6]==1) && (PTab[CurQrsNr-1].MorfT==2))
							ArhMan(4, CurQrsNr-1, 7, 7);   // przedpoprzedni QRS jest VE
						Flags[6]=0;
						}
					}
				}

			/* ===== detekcja epizodow wylacznie nadkomorowych ===== */
			/* =====     SP     ST      SE       SI     ============ */
			else
				{
	/* ===== ocena przedwczesnych pobudzen nadkomorowych ===== */
				if((PTab[CurQrsNr].cl==-8) && (PTab[CurQrsNr].MorfT==1))
					{
					// zespol o morfologii narzuconej przez operatora
					ArhMan(4, CurQrsNr, 1, 1); // oznaczenie zespolu przedwczesnego
					}
				if(PTab[CurQrsNr].cl>-1)
					{
					if (((PTab[CurQrsNr-1].MorfT==0) || (PTab[CurQrsNr-1].MorfT==1)) && 
							((PTab[CurQrsNr+1].MorfT==0) || (PTab[CurQrsNr+1].MorfT==1)) && 
							(Flags[8]==0) && (ClRep[PTab[CurQrsNr].cl][9]))
						// pobudzenia poprzednie i nastepne sa nadkomorowe lub dominujace 
						// i zespol nie nalezy do klasy "o sztywnej morfologii"
						{
						if(SVHR > 0) 
							{
							if(CurQrsNr-LastPauseAt > Cfg[CurCfg].cfga.HRateQRSCnt)
								// jezeli poprzednia pauza nie ma wplywu na wartosc HR
								{
			//procentowe wydluzenie (-) / skrocenie (+) interwalu RR
								INT_S perc=(INT_S)(((100l*SVHR)+(PTab[CurQrsNr].localRR >>1))/PTab[CurQrsNr].localRR)-100;
			/* metoda A */
								if((perc > Cfg[CurCfg].cfga.PremTsh) &&
										(PTab[CurQrsNr+1].localRR > SVHR) &&
										(PTab[CurQrsNr].localRR+PTab[CurQrsNr+1].localRR < (SVHR<<1)))
									{
									ArhMan(4, CurQrsNr, 1, 1); // oznaczenie zespolu przedwczesnego
									PTab[CurQrsNr].MorfT=1;
									}
			/* metoda B */
								else if(perc > (Cfg[CurCfg].cfga.PremTsh<<1))
									{ 
									ArhMan(4, CurQrsNr, 1, 1); // oznaczenie zespolu przedwczesnego
									PTab[CurQrsNr].MorfT=1; 
									}
								}
							}
						}
					}

	/* ===== napadowy czestoskurcz nadkomorowy PSVT ===== */
				if(Flags[8]==0)  //PSVT - detekcja poczatku
					{
					if((CurQrsNr==PrvQrsNr+1) && ((INT_S)(60000l/PTab[CurQrsNr].localRR) > Cfg[CurCfg].cfga.PSVTMinHRtoStart))
						PSVTHRCnt++;
					else
						PSVTHRCnt=0;
					if((PSVTHRCnt > 3) && (SVseq > 3))
						{
						PSVTHRCnt=0;
						CurQrsNr-=3;
						INT_S mhr1=(INT_S)(180000l/(PTab[CurQrsNr].localRR+PTab[CurQrsNr-1].localRR+PTab[CurQrsNr-2].localRR));
						INT_S mhr2=(INT_S)(18000000l/(PTab[CurQrsNr+1].localRR+PTab[CurQrsNr+2].localRR+PTab[CurQrsNr+3].localRR));
						if(((mhr2/mhr1)-100) > Cfg[CurCfg].cfga.PSVTMinAcctoStart)
			// i przyroscie czestosci ponad okreslona wartosc
							{
							INT_L hr10_old=0;
							INT_L hr10_new=0;
							for(i=-9; i<4; i++)
								{
								if(i<1)
									hr10_old+=(60000l/PTab[CurQrsNr+i].localRR);
								if(i>-7)
									hr10_new+=(6000000l/PTab[CurQrsNr+i].localRR);
								}
/*##*/	      if((((INT_S)(hr10_new/hr10_old))-100) > ((/*3**/Cfg[CurCfg].cfga.PSVTMinAcctoStart)>>2))
// sprawdzic warunek przyrostu HR - wydaje sie bardzo silny !!!
				// i przyroscie czestosci sredniej z 10 ponad okreslona wartosc
								{
								for(INT_S i=1; i<4; i++)
									{
									INT_S perc=(int)((100l*PTab[CurQrsNr+i].localRR)/PTab[CurQrsNr+i-1].localRR)-100;
									if(perc < 0)
										perc=-perc;
									if(perc > Cfg[CurCfg].cfga.PSVTMaxITD)
										break;
									}
								if(i==4)
				// petla doszla do konca bez brejkow czyli nierownomiernosc
				// jest mniejsza niz zadana
									{
				//napadowy czestoskurcz nadkomorowy PSVT - poczatek
									ArhMan(1, CurQrsNr, 8, 8);   // open 8
									Flags[8]=10;
									}
								}
							}
						CurQrsNr+=3;
						}
					}
				else  //PSVT - detekcja konca
					{
					if((CurQrsNr==PrvQrsNr+1) && ((INT_S)(60000l/PTab[CurQrsNr].localRR) < Cfg[CurCfg].cfga.PSVTMaxHRtoStop))
						PSVTHRCnt++;
					else
						PSVTHRCnt=0;
					if((PSVTHRCnt > 3) && (SVseq > 3))
						{
						PSVTHRCnt=0;
						CurQrsNr-=3;
						INT_S mhr1=(INT_S)(18000000l/(PTab[CurQrsNr].localRR+PTab[CurQrsNr-1].localRR+PTab[CurQrsNr-2].localRR));
						INT_S mhr2=(INT_S)(180000l/(PTab[CurQrsNr+1].localRR+PTab[CurQrsNr+2].localRR+PTab[CurQrsNr+3].localRR));
						if(((mhr1/mhr2)-100) > Cfg[CurCfg].cfga.PSVTMinDectoStop)
	    // i spadku czestosci ponizej okreslonej wartosci
							{
				// napadowy czestoskurcz nadkomorowy PSVT - koniec
							ArhMan(2, CurQrsNr, 8, 8);   // close 8
							Flags[8]=0;
							}
						CurQrsNr+=3;
						}
					}

	/* ===== poszukiwanie poczatku tachykardii nadkomorowej ===== */
				if(Flags[10]==0)
					{
					if((CurQrsNr==PrvQrsNr+1) && ((INT_S)(60000l/PTab[CurQrsNr].localRR) > Cfg[CurCfg].cfga.SVTMinHRtoStart))
						SVTHRCnt++;
					else
						SVTHRCnt=0;
					if((SVTHRCnt > 2) && (SVseq > 2))
						{
	    // poczatek tachykardii nadkomorowej
						ArhMan(1, CurQrsNr, 10, 10); // close 10
						Flags[10]=10;
						SVTHRCnt=0;
						}
					}

	/* ===== rytm nieregularny ===== */
	//testowanie poczatku rytmu nieregularnego
				if((Flags[8]==0) && (Flags[10]==0) && (Flags[11]==0))
		// jezeli nie ma innych arytmii nadkomorowych
					{
					INT_S perc=(int)((100l*PTab[CurQrsNr+i].localRR)/PTab[CurQrsNr+i-1].localRR)-100;
					if(perc < 0)
						perc=-perc;
					if((CurQrsNr==PrvQrsNr+1) && (perc > Cfg[CurCfg].cfga.IrrDeltaRR))
						IRRCnt++;
					else
						IRRCnt=0;
					if(IRRCnt >=Cfg[CurCfg].cfga.IrrQRSCnt)
						{
			// rytm nieregularny - poczatek
						ArhMan(1, CurQrsNr-Cfg[CurCfg].cfga.IrrQRSCnt, 11, 11); // open 11
						Flags[11]=10;
						IRRCnt=0;
						}
					}
				else if (Flags[11]==10) //testowanie konca rytmu nieregularnego
					{
					INT_S perc=(int)((100l*PTab[CurQrsNr+i].localRR)/PTab[CurQrsNr+i-1].localRR)-100;
					if(perc < 0)
						perc=-perc;
					if((CurQrsNr==PrvQrsNr+1) && (perc < Cfg[CurCfg].cfga.IrrDeltaRR))
						IRRCnt++;
					else
						IRRCnt=0;
					if(IRRCnt >=Cfg[CurCfg].cfga.IrrQRSCnt)
						{
	    // rytm nieregularny - koniec
						ArhMan(2, CurQrsNr-Cfg[CurCfg].cfga.IrrQRSCnt, 11, 11); // close 11
						Flags[11]=0;
						IRRCnt=0;
						}
					}
				}
      }
    else if(PTab[CurQrsNr].MorfT==2)
      // zespol jest komorowy
			{
      VEseq++;
      SVseq=0;
      DBseq=0;
      HRBuffP=0;

      // skasowanie lub zakonczenie ew. arytmii nadkomorowych
      if(Flags[8]==10)
				{
				// napadowy czestoskurcz nadkomorowy PSVT - koniec
				ArhMan(2, CurQrsNr, 8, 8);   // close 8
				Flags[8]=0;
				}
      PSVTHRCnt=0;
      SVTHRCnt=0;
      IRRCnt=0;

      /* ===== bigeminia (czesc B) ===== */
      /* ===== detekcja par NVVN i pojedynczych VE ===== */
      if(VEseq==1)
				{
				if ((PTab[CurQrsNr+1].MorfT==0) || (PTab[CurQrsNr+1].MorfT==1))
					{
					if ((PTab[CurQrsNr+2].MorfT==0) || (PTab[CurQrsNr+2].MorfT==1))
						{
						if(Flags[6]<10) // nie jest to koncowka bigeminii
				// NVNN
	      // pojedynczy zespol komorowy VE
							{
							if((Flags[6]==1) && (PTab[CurQrsNr-2].MorfT==2))
								ArhMan(4, CurQrsNr-2, 7, 7);   // przedpoprzedni QRS jest VE
							ArhMan(4, CurQrsNr, 7, 7);       // biezacy QRS jest VE
							}
						else if (Flags[6]==10)
							ArhMan(2, CurQrsNr, 6, 6); //zakonczenie ew. bigeminii
						Flags[6]=0;
						}
					else if(PTab[CurQrsNr+2].MorfT==2)
		// NVNV
						{
						switch(Flags[6]) // wskaznik bigeminii
							{
							case 0: Flags[6]=1;
								break;
							case 1: Flags[6]=10;
								ArhMan(1, CurQrsNr-2, 6, 6);
								break;
							}
						}
					}
				else if(PTab[CurQrsNr+1].MorfT==2)
					{
					if (Flags[6]==10)
						ArhMan(2, CurQrsNr, 6, 6); //zakonczenie ew. bigeminii
					else
						if ((PTab[CurQrsNr+2].MorfT==0) || (PTab[CurQrsNr+2].MorfT==1))
	  // NVVN
	  // para PR rejestrowana jest tylko wowczas, jeski sekwencja NVVN nie
	  // jest koncem bigeminii
	  // para PR wskaznik CurQrsNr wskazuje na wczesniejszy zespol
							ArhMan(4, CurQrsNr, 5, 5);
					Flags[6]=0;
//	  else if(PTab[CurQrsNr+2].MorfT==2)
//	  // NVVV
					}
				}

			/* ===== detekcja epizodow wylacznie komorowych ===== */
			/* ===========     VT     VR      VS     ============ */
			else
				{
				if(VEseq > Cfg[CurCfg].cfga.VTQRSCnt)
	// minimalna dlugosc sekwencji
					{
					if(PTab[CurQrsNr].HR > Cfg[CurCfg].cfga.VTHRTsh)
		// HR wieksze od progu
						{
						if(VEseq > Cfg[CurCfg].cfga.VTSalvoMax)
			// czestoskurcz komorowy VT (2)
							{
							if (Flags[3]==10) //dotychczas byl: VR
								{
								ArhMan(3, CurQrsNr, 3, 2); //rename 3 to 2
								Flags[3]=0;
								Flags[2]=10;
								}
							if (Flags[4]==10) //dotychczas byl: VS
								{
								ArhMan(3, CurQrsNr, 4, 2); //rename 4 to 2
								Flags[4]=0;
								Flags[2]=10;
								}
							}
						else
				// salvo - pobudzenia gromadne VS (4)
							{
							if (Flags[3]==10) //dotychczas byl: VR
								{
								ArhMan(3, CurQrsNr, 3, 4); //rename 3 to 4
								Flags[3]=0;
								Flags[4]=10;
								}
							if (Flags[4]==0) //dotychczas byl: brak arytmii
								{
								ArhMan(1, CurQrsNr-Cfg[CurCfg].cfga.VTQRSCnt, 4, 4); //open 4
								Flags[4]=10;
								}
							}
						}
					else
	    // czynny rytm komor VR (3)
						{
						if (Flags[2]==10) //dotychczas byl: VE
							{
							ArhMan(3, CurQrsNr, 2, 3); //rename 2 to 3
							Flags[2]=0;
							Flags[3]=10;
							}
						if (Flags[4]==10) //dotychczas byl: VS
							{
							ArhMan(3, CurQrsNr, 4, 3); //rename 4 to 3
							Flags[4]=0;
							Flags[3]=10;
							}
						if (Flags[3]==0) //dotychczas byl: brak arytmii
							{
							ArhMan(1, CurQrsNr-Cfg[CurCfg].cfga.VTQRSCnt, 3, 3); //open 3
							Flags[3]=10;
							}
						}
					} // koniec dla minimalnej dlugosci sekwencji
				}
			} // koniec dla zesp. komorowych
		//koniec petli arytmii
		PrvQrsNr=CurQrsNr;
		}
	/* uzupelnienie struktury ArhStatus */
	ArhStatus.CurQrsNr=CurQrsNr;
	Anuluj_Button= (INT_C)SendMessage(hKernelWnd, AHM_ARYTHM, 1, (LPARAM)&ArhStatus); // (*DW*)

	}

void Rhythm_Ev_Det(INT_L QrsMaxNr)
/* funkcja Rhythm_Ev_Det() realizuje analize tabeli wlasnosci     */
/* zespolow QRS, okresla czestosci rytmu i wystepowanie zjawisk   */
/* o cechach arytmii						  																*/
/* parametry wejsciowe: QrsMaxNr - calkowita ilosc zespolow QRS   */
/* PA 09.12.97 v. 1.08						  															*/
	{
	RrAll(QrsMaxNr);
	ArhyDet(QrsMaxNr);
	MorphReOrd(QrsMaxNr);


//###############
	FILE *wcf=NULL; // plik do ktorego zostanie zapisana tablica ksztatw
	wcf=fopen("qrspars.txt","wt");
	for(INT_L SigPtr=0; SigPtr<QrsMaxNr; SigPtr++)
		{
		fprintf(wcf, "%d\n", ((INT_L)(7.8125*PTab[SigPtr].DetPt)+PTab[SigPtr].AMax)); ///#
		fprintf(wcf, "%d\n",PTab[SigPtr].MorfT);
		fprintf(wcf, "%d\n",PTab[SigPtr].localRR);
		}
	fclose(wcf);
//###############


	ArtfContr(0, QrsMaxNr);
	STEvents(QrsMaxNr);
///  ArhDispl(7);
	}
/* ------------------------------------------------------------------- */
